# File rvsao/Makespec/addspec.x
# September 1, 1999
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 1999 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
 
# ADDSPEC adjusts one spectrum and adds it to a composite spectrum
 
include	<imhdr.h>
include	<imio.h>
include	<fset.h>
include <smw.h>
include "contin.h"
include "rvsao.h"

#  Emission line chopping flags
define  XC_CTYPES       "|no|yes|specfile|"
define  NOCHOP          1       # no correction
define  CHOP            2       # Always remove emission lines
define  SPECFILE        3       # Remove emission lines if spectrum CHOPEM=T

define	LN2		0.69314718

procedure addspec (specfile,specdir,specap,mband,compim,comppath,compspec,
		   ispec,nspec)

char	specfile[ARB]	# Object spectrum file name
char	specdir[ARB]	# Directory for object spectra
int	specap		# Object aperture to read from multispec file
int	mband		# Object band to read from multispec file
pointer compim		# Composite image header structure
char	comppath[ARB]	# Composite spectrum file name
pointer compspec	# Composite spectrum
int	ispec		# Number of spectra combined in this composite so far
int	nspec		# Number of spectra stacked in output file

real	continuum	# Continuum level for output composite spectrum
pointer spectrum	# Object spectrum
pointer specim		# Object image header structure
pointer	work		# Temporary working storage for continuum removal
int	i, j
char	specpath[SZ_PATHNAME]	# Object spectrum path name
char	instrument[SZ_LINE]
double	tz		# Corrections to correlation Z
char	wtitle[20]	# Wavelength axis caption for plots of spectrum
char	title[SZ_LINE]	# Title for plots of spectrum
char	chopstr[SZ_LINE]	# Chop out emission lines
				# (yes | no | specfile)

double	zshift		# Z for composite spectrum
bool	normin,normout	# renormalization flag
char	tstring[SZ_LINE]
char	fstring[SZ_LINE]
char	temp[SZ_LINE]
char	compname[SZ_IMTITLE]	# Title for output spectrum

pointer	shspec		# Rebinned spectrum pixels
pointer	wlcomp		# Wavelength vector for spectrum overlap
pointer	wlspec		# Wavelength vector for complete input spectrum
double 	logmin		# Log-10 of minimum wavelength for cross-correlation
double 	logmax		# Log-10 of maximum wavelength for cross-correlation
double	exp,exp1	# Exposure time in seconds
double	dnormin, dnormout
real 	spmean, speci
double	dindef
real	rindef
double	ra0,ra,dec0,dec
double	dtemp
double	zcomp
double 	sptot
double	spshft		# Spectrum shift in log wavelength
bool	complog		# True if rebinning in log wavelength
double	dlog10()
real	emrej1,abrej1
int	nsp, npts0
char	dstr[SZ_LINE]
double	alt
double	wcs_p2w()
int	npwcs
pointer	wlwcs
pointer	specwcs

bool	clgetb()
real	clgetr()
double	clgetd()
int	clscan()
#int	imaccess()
int	strlen(), ldir, imaccf(), strdic(), strcmp()
pointer sp

define	endspec_	 90
include	"rvsao.com"
include	"contin.com"
include	"sum.com"
 
begin
	dindef = INDEFD
	rindef = INDEFR
	call smark (sp)
	c0 = 299792.5
	specsh = NULL
	call sprintf (wtitle,20,"Wavelength")
	emrej1 = clgetr ("em_reject")
	abrej1 = clgetr ("abs_reject")

# Load spectrum
	ldir = strlen (specdir)
	if (ldir > 0) {
	    if (specdir[ldir] != '/') {
		specdir[ldir+1] = '/'
		specdir[ldir+2] = EOS
		}
	    call strcpy (specdir,specpath,SZ_PATHNAME)
	    call strcat (specfile,specpath,SZ_PATHNAME)
	    }
	else
	    call strcpy (specfile,specpath,SZ_PATHNAME)
	npts0 = npts
	call getspec (specpath, specap, mband, spectrum, specim, FALSE)
	if (specim == ERR) {
	    call eprintf ("ADDSPEC: Error reading spectrum %s\n")
	 	call pargstr (specpath)
	    go to endspec_
	    }
	npts = npts0

# Allocate rebinned spectra and wavelength vectors
	call salloc (shspec, npts, TY_REAL)
	call salloc (wlcomp, npts, TY_REAL)
	call salloc (work, npts, TY_REAL)
	call salloc (wlspec,specpix,TY_REAL)
	do j = 1, specpix {
	    Memr[wlspec+j-1] = real (wcs_p2w (double (j)))
	    }

# Redshift velocity for composite spectrum
	velocity = clgetd ("velcomp")
	zcomp = clgetd ("zcomp")
	if (zcomp != dindef)
	    velocity = c0 * zcomp
	if (debug) {
	    call printf ("ADDSPEC: Z = %.5f, CZ = %.2f\n")
		call pargd (zcomp)
		call pargd (velocity)
	    call flush (STDOUT)
	    }

# If plot enabled, show the object spectrum.
	npwcs = NP2(specsh) - NP1(specsh) + 1
	specwcs = spectrum + NP1(specsh) - 1
	wlwcs = wlspec + NP1(specsh) - 1
	if (pltspec)
	    call plotspec (npwcs,Memr[specwcs],specfile,
			   Memr[wlwcs],wtitle,nsmooth)

# Eliminate bad lines from spectrum
	if (clscan("fixbad") != EOF)
	    call clgstr ("fixbad", fstring, SZ_LINE)
	else
	    call strcpy ("no", fstring)

# Fill in bad lines before removing continuum
	if (strcmp (fstring, "no") != 0 ) {
	    call filllist (npwcs, Memr[specwcs], Memr[wlwcs], debug)
	    if (pltspec) {
		call sprintf (title,SZ_LINE,"%s filling bad lines")
		    call pargstr (specfile)
		call plotspec (npwcs,Memr[specwcs],title,
			       Memr[wlwcs],wtitle,nsmooth)
		}
	    }

# Spectrum line chopping and continuum removal
	emrej[1] = emrej1
	abrej[1] = abrej1
	schop = clgetb ("reject")
	call clgstr ("cont_remove",chopstr,SZ_LINE)
        conproc = strdic (chopstr, chopstr, SZ_LINE, CONT_TYPE)
	if (debug) {
	    if (conproc == DIVCONT)
		call printf ("ADDSPEC: Input spectra continuum divided\n")
	    else if (conproc == SUBCONT)
		call printf ("ADDSPEC: Input spectra continuum subtracted\n")
	    else if (conproc == ZEROCONT)
		call printf ("ADDSPEC: Input spectra continuum divided - 1\n")
	    else
		call printf ("ADDSPEC: Input spectrum continuum not removed\n")
	    }

# Renormalize spectrum if requested
	dnormin = clgetd ("normin")
	if (dnormin <= 0.0)
	    normin = FALSE
	else
	    normin = TRUE
	if (normin) {
	    sptot = 0.d0
	    nsp = 0
	    do i = 1, npwcs {
		speci = Memr[specwcs+i-1]
		if (speci != 0.) {
		    sptot = sptot + speci
		    nsp = nsp + 1
		    }
		}
	    if (nsp > 0) {
		spmean = (sptot / double (nsp)) / dnormin
		do i = 1, specpix {
		    if (Memr[spectrum+i-1] != 0.) {
			Memr[spectrum+i-1] = Memr[spectrum+i-1] / spmean
			}
		    }
		if (debug) {
		    call printf ("ADDSPEC: Spectrum divided by mean = %.2f\n")
			call pargr (spmean)
		    }
		}
	    else {
		call eprintf ("ADDSPEC: Spectrum is all zeroes\n")
		go to endspec_
		}
	    if (pltspec) {
		call sprintf (title,SZ_LINE,"%s renormalized")
		    call pargstr (specfile)
		call plotspec (npwcs,Memr[specwcs],title,
			       Memr[wlwcs],wtitle,nsmooth)
		}
	    }
	tz = 0.d0

# Add known velocity components as 1+z's
	if (spvel != dindef)
	    tz = (1.d0 + (spvel / c0))
	else
	    tz = 1.d0
	if (spechcv != dindef)
	    tz = tz / (1.d0 + (spechcv / c0))

# Set z offset for rebinning
	if (velocity == dindef)
	    zshift = 0.d0
	else {
	    zshift = (1.d0 + (velocity / c0)) / tz
	    }
	if (debug) {
	    if (spvel == dindef)
		call printf ("ADDSPEC: spvel= INDEF, ")
	    else {
		call printf ("ADDSPEC: spvel= %.2f, ")
		    call pargd (spvel)
		}
	    if (spechcv == dindef)
		call printf ("spechcv= INDEF, ")
	    else {
		call printf ("spechcv= %.2f, ")
		    call pargd (spechcv)
		}
	    if (velocity == dindef)
		call printf ("velocity= INDEF, ")
	    else {
		call printf ("velocity= %.2f, ")
		    call pargd (velocity)
		}
	    call printf ("zshift is %.5f\n")
		call pargd (zshift)
	    call flush (STDOUT)
	    }
	if (zshift == 0.d0)
	    spshft = 0.d0
	else
	    spshft = dlog10 (zshift)

# Rebin in log wavelength
	complog = clgetb ("complog")
	if (complog) {

# Figure lambda and log-lambda limits and increment
	    logmin = dlog10 (minwav0)
	    logmax = dlog10 (maxwav0)

# Fill in globally available values of wavelength
	    logw0 = logmin
	    dlogw = (logmax - logmin) / double (npts - 1)

	    if (debug) {
		call printf("ADDSPEC: spectrum %.7f to %.7f = %d %d, %d npts ->%.3f\n")
		    call pargd (spstrt)
		    call pargd (spfnsh)
		    call pargi (SN(specsh))
		    call pargi (DC(specsh))
		    call pargi (npwcs)
		    call pargd (10.d0 ** spshft)
		call flush (STDOUT)
		call printf("ADDSPEC: binned to %.7f to %.7f by %.9f, %d pts\n")
		    call pargd (logmin)
		    call pargd (logmax)
		    call pargd (dlogw)
		    call pargi (npts)
		call flush (STDOUT)
		}

#  Rebin spectrum to match desired region
	    call rebinl (Memr[spectrum],specsh, spshft, Memr[shspec], npts,
		     logw0, dlogw)

# Set up wavelength vector for intermediate graphs
	    do j = 1, npts {
		Memr[wlcomp+j-1] = 10 ** (logw0 + (j-1) * dlogw)
		}
	    call sprintf (title,SZ_LINE,"%s rebinned in log wavelength")
		call pargstr (specfile)
	    }

# Rebin in linear wavelength
	else {
	    dw = (maxwav0 - minwav0) / double (npts - 1)
	    if (debug) {
		call printf("ADDSPEC: spectrum %.3fA to %.3fA = %d %d, %d npts ->%.3f\n")
		    call pargd (10.d0 ** spstrt)
		    call pargd (10.d0 ** spfnsh)
		    call pargi (SN(specsh))
		    call pargi (DC(specsh))
		    call pargi (npwcs)
		    call pargd (10.d0 ** spshft)
		call flush (STDOUT)
		call printf("ADDSPEC: binned to %.3fA to %.3fA by %.3fA, %d pts\n")
		    call pargd (minwav0)
		    call pargd (maxwav0)
		    call pargd (dw)
		    call pargi (npts)
		call flush (STDOUT)
		}

#  Rebin spectrum to match desired region
	    call rebin (Memr[spectrum],specsh, spshft, Memr[shspec], npts,
		     minwav0, dw)

# Set up wavelength vector for intermediate graphs
	    do j = 1, npts {
		Memr[wlcomp+j-1] = minwav0 + (double (j-1) * dw)
		}
	    call sprintf (title,SZ_LINE,"%s rebinned in wavelength")
		call pargstr (specfile)
	    }

# Renormalize rebinned spectrum if requested
	dnormout = clgetd ("normout")
	if (dnormout <= 0.0)
	    normout = FALSE
	else
	    normout = TRUE
	if (normout) {
	    sptot = 0.d0
	    nsp = 0
	    do i = 1, npts {
		speci = Memr[shspec+i-1]
		if (speci != 0.) {
		    sptot = sptot + speci
		    nsp = nsp + 1
		    }
		}
	    if (nsp > 0) {
		spmean = (sptot / double (nsp)) / dnormout
		do i = 1, npts {
		    if (Memr[shspec+i-1] != 0.) {
			Memr[shspec+i-1] = Memr[shspec+i-1] / spmean
			}
		    }
		if (debug) {
		    call printf ("ADDSPEC: Rebinned spectrum divided by mean = %.2f\n")
			call pargr (spmean)
		    }
		}
	    else {
		call eprintf ("ADDSPEC: Rebinned spectrum is all zeroes\n")
		go to endspec_
		}
	    }

# Plot rebinned spectrum, if requested
	if (pltspec)
	    call plotspec (npts,Memr[shspec],title,Memr[wlcomp],wtitle,nsmooth)

	call logtime (dstr,SZ_LINE)

# If output file is not yet open, open it
	if (compim == NULL) {
	    if (nspec == 1 || nmspec > 1)
		call tmp_open (compim, comppath, compspec, npts, nspec, specim)
	    else
		call tmp_open (compim, comppath, compspec, npts, nspec, NULL)
	    if (compim == ERR) {
		call eprintf ("ADDSPEC: Error writing composite spectrum %s\n")
	 	    call pargstr (comppath)
		go to endspec_
		}
	    call clgstr ("compname",compname,SZ_IMTITLE)
	    if (strlen (compname) <= 0)
		call strcpy (IM_TITLE(specim),IM_TITLE(compim),SZ_IMTITLE)
	    else
		call strcpy (compname,IM_TITLE(compim),SZ_IMTITLE)

	    ra0 = dindef
	    call strcpy ("0:00:00", tstring, SZ_LINE)
	    call imcpsd (specim, compim, "RA", ra0)

	    dec0 = dindef
	    call strcpy ("0:00:00", tstring, SZ_LINE)
	    call imcpsd (specim, compim, "DEC", dec0)

	    dtemp = 1950.0
	    call imcpd (specim, compim, "EPOCH", dtemp)
	    call imcpd (specim, compim, "EQUINOX", dtemp)
	    call imcpstr (specim, compim, "INSTRUME", tstring)
	    call imcpstr (specim, compim, "SITENAME", tstring)
	    call imcpstr (specim, compim, "SITELONG", tstring)
	    call imcpstr (specim, compim, "SITELAT", tstring)
	    alt = 0.d0
	    call imcpd (specim, compim, "SITEELEV", alt)

	    exp = 0.d0
	    call imgdpar (specim, "EXPTIME",exp)
	    if (exp .le. 0.d0)
		call imgdpar (specim, "EXPOSURE", exp)
	    if (exp .le. 0.d0)
		call imgdpar (specim, "ITIME", exp)
	    if (exp < .001d0)
		exp = 1.d0
	    call imaddd (compim, "EXPTIME", exp)
	    texp = exp
	    call imaddi (compim, "DISPAXIS", 1)

	    if (conproc == DIVCONT)
		call sprintf (tstring, SZ_LINE, "Cont div, %s order %d, %d iter  %d -%d")
	    else if (conproc == ZEROCONT)
		call sprintf (tstring, SZ_LINE, "Cont div-1, %s order %d, %d iter  %d -%d")
	    else if (conproc == SUBCONT)
		call sprintf (tstring, SZ_LINE, "Cont sub, %s order %d, %d iter -%d %d")
	    if (conproc != NOCONT) {
		if (function == CN_SPLINE3)
		    call pargstr ("spline3")
		else if (function == CN_LEGENDRE)
		    call pargstr ("legendre")
		else if (function == CN_CHEBYSHEV)
		    call pargstr ("chebyshev")
		else
		    call pargstr ("spline1")
		call pargi (order)
		call pargi (niterate)
		call pargr (lowrej[1])
		call pargr (hirej[1])
		if (normin || normout)
		    call strcat (" normalized", tstring, SZ_LINE)
		call imastr (compim, "HISTORY", tstring);
		}
	    if (complog) {
		call imaddi (compim, "DC-FLAG", 1)
		call imaddr (compim, "CRPIX1", 1.)
		call imaddd (compim, "CRVAL1", logw0)
		call imaddd (compim, "CDELT1", dlogw)
		call imaddd (compim, "CD1_1", dlogw)
		}
	    else {
		call imaddi (compim, "DC-FLAG", 0)
		call imastr (compim, "CTYPE1","LINEAR")
		call imaddr (compim, "CRPIX1", 1.)
		call imaddd (compim, "CRVAL1", minwav0)
		call imaddd (compim, "CDELT1", dw)
		call imaddd (compim, "CD1_1", dw)
		call imastr (compim, "CTYPE2","LINEAR")
		call imaddi (compim, "CRPIX2", 1)
		call imaddi (compim, "CRVAL2", 1)
		call imaddi (compim, "CDELT2", 1)
		call imaddi (compim, "CD2_2", 1)
		}

	    call imaddi (compim, "NP1", 1)
	    call imaddi (compim, "NP2", npts)
	    if (velocity != dindef) {
		call imaddd (compim, "VELOCITY", velocity)
		call imaddd (compim, "BCV", 0.d0)
		}
	    call sprintf (tstring,SZ_LINE,"RVSAO.SUMSPEC 2.0 run %s")
		call pargstr (dstr)
	    call imputh (compim, "HISTORY", tstring)
	    }

	if (ispec == 1) {

	# Subtract continuum level from input spectrum
	    if (conproc != NOCONT)
		call icsubcon (npts,Memr[shspec],Memr[wlcomp],specname,1,
			       nsmooth,Memr[work])

	    if (strcmp (fstring, "zero") == 0) {
		call zerolist (npts, Memr[shspec], Memr[wlcomp], debug)
		if (pltspec) {
		    call sprintf (title,SZ_LINE,"%s zeroing bad lines")
			call pargstr (specfile)
		    call plotspec (npts,Memr[shspec],title,
				   Memr[wlcomp],wtitle,nsmooth)
		    }
		}

	# Move input spectrum to composite spectrum
	    call amovr (Memr[shspec],Memr[compspec],npts)

	# Add continuum level to composite spectrum
	    continuum = clgetr ("cont_add")
	    if (continuum != 0.0)
		call aaddkr (Memr[compspec], continuum, Memr[compspec], npts)

	# Enter in log files
	    do i = 1, nlogfd {
		call fprintf (logfd[i],"ADDSPEC %s making %s\n")
		    call pargstr (dstr)
		    call pargstr (comppath)
		call fprintf (logfd[i],"ADDSPEC %d: %s")
		    call pargi (ispec)
		    call pargstr (specpath)
		if (specap > 0) {
		    if (mband > 0) {
			call fprintf (logfd[i],"[%d,%d]")
			    call pargi (specap)
			    call pargi (mband)
			}
		    else {
			call fprintf (logfd[i],"[%d]")
			    call pargi (specap)
			}
		    }
		if (normin) {
		    call fprintf (logfd[i]," * %g10")
			call pargr (spmean)
		    }
		call fprintf (logfd[i]," added")
		if (velocity != dindef && spechcv != 0.d0 && spechcv != dindef){
		    if (spvel != dindef) {
			call fprintf (logfd[i],", shifted %.2f - %.2f")
			    call pargd (spvel)
			    call pargd (spechcv)
			}
		    else {
			call fprintf (logfd[i],", shifted - %.2f")
			    call pargd (spechcv)
			}
		    }
		else if (velocity != dindef)
			call fprintf (logfd[i],", shifted")

		if (velocity != dindef) {
		    call fprintf (logfd[i]," to %.2f")
			call pargd (velocity)
		    }
		call fprintf (logfd[i],"\n")
		}
	    }

	else {

	# Compare positions; if different from last, delete position from header
	    if (imaccf (specim, "RA") == YES) {
		call imgstr (specim, "RA", tstring,SZ_LINE)
		call sscan (tstring)
		    call gargd (ra)
		}
	    else
		ra = ra0
	    if (imaccf (specim, "DEC") == YES) {
		call imgstr (specim, "DEC", tstring,SZ_LINE)
		call sscan (tstring)
		    call gargd (dec)
		}
	    else
		dec = dec0
	    if (ra != ra0 || dec != dec0) {
		if (imaccf (compim, "RA") == YES)
		    call imdelf (compim, "RA")
		if (imaccf (compim, "DEC") == YES)
		    call imdelf (compim, "DEC")
		if (imaccf (compim, "EPOCH") == YES)
		    call imdelf (compim, "EPOCH")
		if (imaccf (compim, "EQUINOX") == YES)
		    call imdelf (compim, "EQUINOX")
		}

	# Add this exposure time and put back into the output header
	    exp1 = 0.d0
	    call imgdpar (specim, "EXPTIME",exp1)
	    if (exp1 .le. 0.d0)
		call imgdpar (specim, "EXPOSURE", exp1)
	    if (exp1 .le. 0.d0)
		call imgdpar (specim, "ITIME", exp1)
	    if (exp1 < .001d0)
		exp1 = 1.d0

	# If non-zero exposure time, add this spectrum
	    if (exp1 > 0.d0) {
		if (conproc != NOCONT)
		    call icsubcon (npts,Memr[shspec],Memr[wlcomp],specname,1,
				   nsmooth,Memr[work])

		if (strcmp (fstring, "zero") == 0) {
		    call zerolist (npts, Memr[shspec], Memr[wlcomp], debug)
		    if (pltspec) {
			call sprintf (title,SZ_LINE,"%s zeroing bad lines")
			    call pargstr (specfile)
			call plotspec (npts,Memr[shspec],title,
					Memr[wlcomp],wtitle,nsmooth)
			}
		    }

		if (nspec > 1)
		    call amovr (Memr[shspec],Memr[compspec],npts)
		else
		    call aaddr (Memr[compspec],Memr[shspec],Memr[compspec],npts)
		exp = texp + exp1
		if (nspec > 1)
		    exp = exp / double (nspec)
		call imaddd (compim, "EXPTIME", exp)

	# Enter in log files
		do i = 1, nlogfd {
		    call fprintf (logfd[i],"ADDSPEC %d: %s")
			call pargi (ispec)
			call pargstr (specpath)
		    if (specap > 0) {
			if (mband > 0) {
			    call fprintf (logfd[i],"[%d,%d]")
				call pargi (specap)
				call pargi (mband)
			    }
			else {
			    call fprintf (logfd[i],"[%d]")
				call pargi (specap)
			    }
			}
		    if (normin) {
			call fprintf (logfd[i]," * %g10")
			    call pargr (spmean)
			}
		    call fprintf (logfd[i]," added")
		    if (velocity != dindef && spechcv != 0.d0 && spechcv != dindef){
			if (spvel != dindef) {
			    call fprintf (logfd[i],", shifted %.2f - %.2f")
				call pargd (spvel)
				call pargd (spechcv)
			    }
			else {
			    call fprintf (logfd[i],", shifted - %.2f")
				call pargd (spechcv)
			    }
			}
		    else if (velocity != dindef)
			call fprintf (logfd[i],", shifted")

		    if (velocity != dindef) {
			call fprintf (logfd[i]," to %.2f")
			    call pargd (velocity)
			}
		    if (ispec >= nspec)
			call fprintf (logfd[i],"\n")
		    else
			call fprintf (logfd[i],"\r")
		    }
		}
	    else {
		do i = 1, nlogfd {
		    call fprintf (logfd[i],"ADDSPEC: %s skipped EXP is %.2f\n")
			call pargstr (specpath)
			call pargd (exp1)
		    }
		}
	    }

	if (ispec == 1 && normin)
	    call imaddr (compim, "SPNORM", spmean)

# Note addition of this spectrum to the composite spectrum
	call sprintf (tstring,SZ_LINE,"Add %s %s")
	    call pargstr (instrument)
	    call pargstr (specpath)
	if (specap > 0) {
	    if (mband > 0) {
		call sprintf (temp,SZ_LINE,"[%d,%d]")
		    call pargi (specap)
		    call pargi (mband)
		}
	    else {
		call sprintf (temp,SZ_LINE,"[%d]")
		    call pargi (specap)
		}
	    call strcat (temp, tstring, SZ_LINE)
	    }
	if (spvel != dindef || (spechcv != dindef && spechcv != 0.d0)) {
	    if (velocity == dindef)
		call strcat (" at indef CZ",tstring, SZ_LINE)
	    else {
		call strcat (" at CZ",tstring, SZ_LINE)
		if (spvel != dindef) {
		    call sprintf (temp,SZ_LINE," %.2f")
			call pargd (spvel)
		    call strcat (temp, tstring, SZ_LINE)
		    }
		if (spechcv != dindef && spechcv != 0.d0) {
		    call sprintf (temp,SZ_LINE," %.2f")
			call pargd (spechcv)
		    call strcat (temp, tstring, SZ_LINE)
		    }
		}
	    }

	if (save_names)
	    call imputh (compim, "HISTORY", tstring)

# Check for writability
#	if (imaccess (comppath, WRITE_ONLY) == NO) {
#	    call eprintf ("ADDSPEC: cannot write to %s; not saving results\n")
#		call pargstr (comppath)
#	    IM_UPDATE(compim) = NO
#	    }
#	else
	    IM_UPDATE(compim) = YES

# If plot enabled, show the current composite spectrum
	if (plttemp) {
	    if (ispec > 1) {
		call sprintf (title,SZ_LINE,"%s using %d spectra")
		    call pargstr (compname)
		    call pargi (ispec)
		}
	    else if (strlen (compname) > 0) {
		call sprintf (title,SZ_LINE,"%s using %s")
		    call pargstr (compname)
		    call pargstr (specfile)
		}
	    else {
		call sprintf (title,SZ_LINE,"%s using %s")
		    call pargstr (comppath)
		    call pargstr (specfile)
		}
	    call plotspec (npts,Memr[compspec],title,
		       Memr[wlcomp],wtitle,tsmooth)
	    }

# Free rebinned spectra, wavelength, and correlation vectors
endspec_
	call sfree (sp)

# Close the object spectrum image
	IM_UPDATE(specim) = NO
	call close_image (specim, specsh)
	return

end


procedure imcpstr (imin, imout, keyword, tstring)

pointer imin		# Input image header structure
pointer imout		# Output image header structure
char	keyword[ARB]	# Keyword to be transferred
char	tstring[SZ_LINE] # value of keyword (returned)

int	imaccf()
begin
	if (imaccf (imin, keyword) == YES) {
	    call imgstr (imin, keyword, tstring,SZ_LINE)
	    call imastr (imout, keyword, tstring)
	    }
	return
end


procedure imcpsd (imin, imout, keyword, dtemp)

pointer imin		# Input image header structure
pointer imout		# Output image header structure
char	keyword[ARB]	# Keyword to be transferred
double	dtemp		# value of keyword (returned)

char	tstring[SZ_LINE] # value of keyword (returned)
int	imaccf()
begin
	if (imaccf (imin, keyword) == YES) {
	    call imgstr (imin, keyword, tstring,SZ_LINE)
	    call imastr (imout, keyword, tstring)
	    call sscan (tstring)
		call gargd (dtemp)
	    }
	return
end


procedure imcpd (imin, imout, keyword, dtemp)

pointer imin		# Input image header structure
pointer imout		# Output image header structure
char	keyword[ARB]	# Keyword to be transferred
double	dtemp		# value of keyword (returned)

int	imaccf()
begin
	if (imaccf (imin, keyword) == YES) {
	    call imgdpar (imin, keyword, dtemp)
	    call imaddd (imout, keyword, dtemp)
	    }
	return
end

# Jul 21 1995	New program
# Oct  6 1995	Do not add spectrum to composite unless exposure time is > 0
# Oct 11 1995	Add continuum removal

# Jan 10 1997	Change parameter names to show that em and abs lines are cut
# Jan 14 1997	Change tempvel to veltemp
# Jan 14 1997	Adjust continuum processing to current RVSAO methods
# Feb  3 1997	Drop unused parameter t_chop
# Feb  4 1997	Drop declaration of C0; it is now in rvsao.com
# Feb  4 1997	Fix calls to plotspec
# Feb  5 1997	Fix HISTORY statement; fix EPOCH transfer
# Mar 14 1997	Drop SPECSH from GETSPEC call; more it to common
# Apr 18 1997	Fix unrebinned graph of input spectra
# Apr 21 1997	Move wavelength limit setting to T_SUMTEMP
# Apr 21 1997	Add barycentric velocity correction to HISTORY
# Apr 25 1997	Add composite title from parameter tempobj
# Apr 25 1997	Add composite continuum from parameter continuum
# Apr 29 1997	Change name from ADDTEMP to ADDSPEC and variable names, too
# May  1 1997	Include SMW.H instead of SHDR.H
# May  2 1997	Fix title in composite graph
# May  2 1997	Always test against dindef, not INDEFD
# May  5 1997	Drop S_ from REJECT parameters and change SCHOP to REJECT
# May  5 1997	Change S_CONT parameter to CONT_REMOVE
# May  5 1997	Change CONTINUUM parameter to CONT_ADD
# May  6 1997	Drop position from header if different objects added
# May  6 1997	Add NSMOOTH argument to ICSUBCON
# May 16 1997	Add option to rebin to linear wavelength
# May 16 1997	If VELOCITY is INDEF, do not shift or write to header
# May 21 1997	Add renormalization after continuum removal and rebinning
# May 21 1997	Fix option to rebin to linear wavelength; improve logging
# Jun  5 1997	Add FIXBAD and BADLINES to remove specific wavelength regions
# Jun 11 1997	Add end of line to log listing
# Jun 17 1997	Clean up handling of undefined velocities
# Jul 21 1997	Reset NPTS so it isn't change by the value in the input header
# Jul 22 1997	Make logging of input filenames in header optional
# Jul 22 1997	Move normalization from before to after continuum removal
# Aug 27 1997	Add multspec band as argument
# Dec 17 1997   Use EQUINOX if it is present as well as EPOCH
#
# Apr 17 1998	Change ctype from pixel to linear
# Apr 22 1998	Drop getim.com; extract needed parameters from header locally
# Jun 12 1998	Use only portions of spectra with WCS
# Dec 16 1998	Write normalization factor for first spectrum
# Dec 16 1998	Write indefinite velocity to history if velocity not being used
# Dec 18 1998	Add option to rebin 2D spectrum to 2D spectrum

# Mar 11 1999	Add write argument to getspec()
# Mar 18 1999	When renormalizing summed string, multiply by 1000
# Mar 19 1999	Fix logic for multiple spectrum summing
# May 11 1999	Suppress line feeds if adding multiple spectra to a file
# May 11 1999	Check for output file WRITE_ONLY, not READ_WRITE
# May 11 1999	Fix spectrum stacking
# Jun 10 1999	Add header HISTORY line for continuum removal
# Jun 10 1999	Keep average exposure if stacking spectra rather than adding
# Jun 10 1999	Add option of zeroing bad lines after continuum removal
# Jun 16 1999	Add normalization to HISTORY
# Jul 27 1999	Add option to copy entire input header to output file
# Jul 28 1999	Change normalization parameters to numbers
# Aug 18 1999	Add divide and subtract one continuum removal option
# Sep  1 1999	Set CD1_1 as well as CDELT1
